import { LocalProxy, LocalProxyOpts } from './index.ts';
import http from 'node:http';
import { fileProxy } from './proxy/file-proxy.ts';
const localProxy = new LocalProxy({});
let home = '/root/center';
export const initProxy = (data: LocalProxyOpts & { home?: string }) => {
  localProxy.pagesDir = data.pagesDir || '';
  localProxy.watch = data.watch ?? false;
  localProxy.init();
  home = data.home;
};

export const proxyRoute = async (req: http.IncomingMessage, res: http.ServerResponse) => {
  const url = new URL(req.url, 'http://localhost');
  const pathname = decodeURIComponent(url.pathname);
  if (pathname === '/') {
    res.writeHead(302, { Location: home + '/' });
    return res.end();
  }
  if (pathname.startsWith('/favicon.ico')) {
    res.statusCode = 404;
    res.end('Not Found Favicon');
    return;
  }
  if (pathname.startsWith('/client') || pathname.startsWith('/api') || pathname.startsWith('/v1') || pathname.startsWith('/serve')) {
    console.debug('handle by router');
    return;
  }
  const localProxyProxyList = localProxy.getLocalProxyList();
  const localProxyProxy = localProxyProxyList.find((item) => pathname.startsWith(item.path));
  if (localProxyProxy) {
    return fileProxy(req, res, {
      path: localProxyProxy.path,
      rootPath: localProxy.pagesDir,
      indexPath: localProxyProxy.indexPath,
    });
  }
  res.statusCode = 404;
  res.end('Not Found Proxy');
};
